package be.iminds.ilabt.jfed.restapi.client;

import be.iminds.ilabt.jfed.fedmon.webapi.client.FedmonWebApiClientConfig;
import be.iminds.ilabt.jfed.fedmon.webapi.client.FedmonWebApiClientPropertiesConfig;
import be.iminds.ilabt.jfed.fedmon.webapi.service.json.FedmonApiCommon;
import be.iminds.ilabt.jfed.fedmon.webapi.service.json.JsonLdObjects;
import be.iminds.ilabt.jfed.restapi.json.Experiment;
import be.iminds.ilabt.jfed.restapi.json.SpeaksForCredential;
import be.iminds.ilabt.jfed.restapi.json.User;
import be.iminds.ilabt.jfed.util.JFedTrustStore;
import org.apache.http.client.utils.URIBuilder;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.net.URISyntaxException;
import java.security.PrivateKey;
import java.security.cert.X509Certificate;
import java.util.List;
import java.util.Properties;

public class JFedRestApiClientPropertiesConfig implements JFedRestApiClientConfig {
    private static final Logger LOG = LoggerFactory.getLogger(FedmonWebApiClientPropertiesConfig.class);

    private final Properties properties;
    private final JFedTrustStore jFedTrustStore;
    private final List<X509Certificate> userCertificate;
    private final PrivateKey userPrivateKey;

    public JFedRestApiClientPropertiesConfig(Properties properties, JFedTrustStore jFedTrustStore, List<X509Certificate> userCertificate, PrivateKey userPrivateKey) {
        this.properties = properties;
        this.jFedTrustStore = jFedTrustStore;
        this.userCertificate = userCertificate;
        this.userPrivateKey = userPrivateKey;
    }

    public String getProperty(String key) {
        return properties.getProperty(key);
    }

    private String getBaseUrl(boolean read) {
        String baseUrl;
        if (read) {
            baseUrl = properties.getProperty("jfed_restapi_client_url_read_base");
            if (baseUrl == null) {
                baseUrl = "https://localhost/";
                LOG.warn("webapi_client_url_read_base config option not set. Falling back to default: "+baseUrl);
            }
        } else {
            baseUrl = properties.getProperty("jfed_restapi_client_url_write_base");
            if (baseUrl == null) {
                baseUrl = "https://localhost/";
                LOG.warn("webapi_client_url_write_base config option not set. Falling back to default: "+baseUrl);
            }
        }
        return baseUrl;
    }

    @Override
    public URIBuilder getUriBuilder(Class<? extends JsonLdObjects.BasicLdObject> objectClass, boolean read) {
        return getUriBuilder(objectClass, "", read);
    }

    @Override
    public URIBuilder getUriBuilder(Class<? extends JsonLdObjects.BasicLdObject> objectClass, String pathEnd, boolean read) {
        if (pathEnd == null)
            pathEnd = "";

        String urlObjectName = null;//FedmonApiCommon.getUrlObjectName(objectClass);
        if (objectClass.equals(Experiment.class))
            urlObjectName = "experiment";
        if (objectClass.equals(SpeaksForCredential.class))
            urlObjectName = "speaksfor";
        if (objectClass.equals(User.class))
            urlObjectName = "user";
        assert urlObjectName != null;

        String baseUrl = getBaseUrl(read);

        URIBuilder builder = null;
        try {
            builder = new URIBuilder(baseUrl);
            String basePath = ("/"+builder.getPath()+"/").replaceAll("//", "/");
            String fullPath = basePath + urlObjectName + "/" + pathEnd;
            builder.setPath(fullPath);
        } catch (URISyntaxException e) {
            throw new RuntimeException("Invalid URL in configuration: \""+baseUrl+"\"", e);
        }

        return builder;
    }

    @Override
    public JFedTrustStore getTrustStore() {
        return jFedTrustStore;
    }

    @Override
    public List<X509Certificate> getUserCertificateChain() {
        return userCertificate;
    }

    @Override
    public PrivateKey getUserPrivateKey() {
        return userPrivateKey;
    }
}
